module MongoMapper
  module ReadonlyAttributes
    extend ActiveSupport::Concern

    included do
	  class_attribute :_attr_readonly, instance_accessor: false
	  self._attr_readonly = []
	  validate :attr_unchanged, on: :update
		
	  def attr_unchanged
	    self._attr_readonly.each do |attr|
		  unless self[attr] == collection.find_one(:_id => self.id)[attr]
		    errors.add(attr, 'cannot be changed')
		  end
	    end
      end
    end

    module ClassMethods
	  def attr_readonly(*attributes)
	    self._attr_readonly = Set.new(attributes.map { |a| a.to_s }) + (self._attr_readonly || [])
	  end

	  def readonly_attributes
		self._attr_readonly
	  end
	end
  end
end